@@ -1,6 +1,6 @@
 """config.py
 
-Keeps tabs of all settings.
+Keep tabs of all settings.
 """
 import sys
 import types
@@ -274,38 +274,56 @@ class _State:
             info = self.friendly_info[-1].info
             info["lang"] = self.lang
             self.saved_info.append(info)
-        except Exception as e:  # pragma: no cover
+        except Exception:  # pragma: no cover
             if not debug_helper.DEBUG:
                 print(
                     "Exception raised by friendly-traceback. Please report this case."
                 )
+                return {}
             debug_helper.log("Exception raised in get_traceback_info().")
-            try:
-                debug_helper.log(self.friendly_info[-1].tb_data.filename)
-            except Exception:  # noqa
-                pass
-            debug_helper.log_error(e)
-            return {}
+            raise
         return info
 
 
 session = _State()
 # It might sometimes be useful to import Friendly-traceback after an
 # exception occurred.
-try:
-    if sys.last_type is not None:
+
+
+def did_exception_occur_before() -> bool:
+    """If an exception occurred before friendly-traceback was imported, it captures
+    the information and prints an informative message to the user.
+
+    Returns True if such information was captures, False otherwise.
+    """
+    # Note that if an exception occurred while friendly-traceback was imported,
+    # this will not capture the information.
+    # For example, suppose we try to do:
+    #
+    #    from friendly.python import *
+    #
+    # This will generate a traceback with no information being captured.
+    # By calling did_exception_occur_before(), we might be able to see if an exception
+    # had been raised and that there is some information available.
+    def _exception_occurred():
+        return _(
+            "An exception occurred before friendly-traceback was imported.\n"
+            "Some information is available."
+        )
+
+    if (
+        hasattr(sys, "last_type")
+        and hasattr(sys, "last_type")
+        and hasattr(sys, "last_traceback")
+    ):
         _info = session.get_traceback_info(
             sys.last_type, sys.last_value, sys.last_traceback
         )
         if _info:
-            session.exception_before_import = True
             session.sys_last_type = sys.last_type
             session.sys_last_value = sys.last_value
-            print(
-                _(
-                    "An exception occurred before friendly-traceback was imported.\n"
-                    "Some information is available."
-                )
-            )
-except AttributeError:  # sys.last_type only exists if an exception has been raised.
-    pass
+            if not session.exception_before_import:
+                print(_exception_occurred())
+            session.exception_before_import = True
+            return True
+    return False
