@@ -423,11 +423,14 @@ class FriendlyTraceback:
         """Determine the cause of an exception, which is what is returned
         by ``why()``.
         """
-        if self.tb_data.filename == "<unknown>" or (
-            self.tb_data.filename == "<string>"
-            and self.tb_data.value.lineno != 1
-            and not issubclass(self.tb_data.exception_type, SyntaxError)
-        ):
+        try:
+            if self.tb_data.filename == "<unknown>" or (
+                self.tb_data.filename == "<string>"
+                and self.tb_data.value.lineno != 1
+                and not issubclass(self.tb_data.exception_type, SyntaxError)
+            ):
+                return
+        except Exception:
             return
 
         if STR_FAILED in self.message:
@@ -537,7 +540,7 @@ class FriendlyTraceback:
         self.locate_exception_raised(records[-1])
         self.info["detailed_tb"] = self.get_detailed_stack_info(records)
         if len(records) > 1:
-            _ignore, partial_source, var_info = self.info["detailed_tb"][0]
+            _ignore, partial_source, var_info = self.info["detailed_tb"][-1]
             self.locate_last_call(records[0], partial_source, var_info)
 
     def locate_exception_raised(self, record: FrameInfo) -> None:
