@@ -17,8 +17,8 @@ from typing import Any, Callable, Mapping, Optional, Union
 import friendly_traceback
 
 from . import source_cache
-from .config import session
-from .console_helpers import helpers
+from .config import did_exception_occur_before, session
+from .console_helpers import friendly_tb, helpers
 from .ft_gettext import current_lang
 from .typing_info import Formatter, InclusionChoice
 
@@ -60,7 +60,7 @@ class FriendlyTracebackConsole(InteractiveConsole):
         ipython_prompt: bool = True,
     ) -> None:
         """This class builds upon Python's code.InteractiveConsole
-        so as to provide friendly tracebacks. It keeps track
+        to provide friendly tracebacks. It keeps track
         of code fragment executed by treating each of them as
         an individual source file.
         """
@@ -76,6 +76,15 @@ class FriendlyTracebackConsole(InteractiveConsole):
             sys.ps1 = "[1]: "
             sys.ps2 = "...: "
         super().__init__(locals=local_vars)
+        session.suggest_console = ""
+
+    def interact(self, banner=None, exitmsg=None) -> None:
+        if not session.exception_before_import:
+            if did_exception_occur_before():
+                print(banner)
+                banner = ""
+                friendly_tb()
+        super().interact(banner=banner, exitmsg=exitmsg)
 
     def push(self, line: str) -> bool:
         """Push a line to the interpreter.
@@ -95,10 +104,10 @@ class FriendlyTracebackConsole(InteractiveConsole):
         source = "\n".join(self.buffer)  # type: ignore
 
         # Each valid code sample is saved with its own fake filename.
-        # They are numbered consecutively to help understanding
+        # They are numbered consecutively to help understand
         # the traceback history.
         # If self.counter was not updated, it means that the previous
-        # code sample was not valid and we reuse the same file name
+        # code sample was not valid, and we reuse the same file name
         filename = self.fake_filename % self.counter
         source_cache.cache.add(filename, source)
 
@@ -161,8 +170,7 @@ class FriendlyTracebackConsole(InteractiveConsole):
         display a traceback.  All exceptions are caught except
         SystemExit, which, unlike the case for the original version in the
         standard library, cleanly exists the program. This is done
-        so as to avoid our Friendly's exception hook to intercept
-        it and confuse the users.
+        to avoid our Friendly's exception hook to intercept it and confuse the users.
 
         A note about KeyboardInterrupt: this exception may occur
         elsewhere in this code, and may not always be caught.  The
