@@ -127,9 +127,6 @@ def too_many_values_to_unpack(
     return {"cause": cause}
 
 
-# TODO: complete the work below; note noqa to be removed
-
-
 @parser.add
 def invalid_literal_for_int(
     message: str, _frame: FrameType, _tb_data: TracebackData
@@ -162,6 +159,8 @@ def invalid_literal_for_int(
             convert = unicodedata.numeric(char, None)
             if convert is not None:
                 convert = str(int(convert))
+            else:
+                convert = char
         if convert not in valid and char not in invalid:
             invalid.append(char)
     invalid = _("The following characters are not allowed: `{invalid}`.\n").format(
