@@ -15,6 +15,11 @@ from .syntax_utils import matching_brackets
 # neighbours, the code for the analysis can be greatly simplified as we do
 # not have to verify the existence of these neighbours.
 MEANINGLESS_TOKEN = token_utils.tokenize(" ")[0]
+# fmt: off
+LINE_NUMBER        = "       {:%d}: "  # noqa
+MARKED_LINE_NUMBER = "    -->{:%d}: "  # noqa
+LINE_GAP           = "       (...)"  # noqa
+# fmt: on
 
 
 class Statement:
@@ -29,12 +34,12 @@ class Statement:
     (), [], {}. If a problem arises due to non-matching pairs of brackets,
     this information is available (variables begin_brackets or end_bracket).
 
-    A complete statement is saved as a list of tokens (statement_tokens)
+    A complete statement is saved as a list of tokens (self.statement_tokens)
     from which it could be reconstructed using an untokenize function.
 
-    From this list of tokens, a secondary list is obtained (tokens) by
-    removing all space-like and comment tokens, which are the only
-    meaningful tokens needed to do the analysis.
+    From this list of tokens, a secondary list is obtained (self.tokens) by
+    removing all space-like and comment tokens; this secondary list
+    thus includes the only meaningful tokens needed to do the analysis.
 
     To simplify the code doing the error analysis itself, we precompute various
     parameters (e.g. does the statement fits on a single line, how many
@@ -48,10 +53,11 @@ class Statement:
         self.linenumber = value.lineno
         self.message = value.msg
         self.offset = value.offset
+        self.value = value  # can be useful when debugging with the repl
 
         # Python 3.10 introduced new attributes for 'value'.
         # We already have taken care of assigning some default values
-        # to it in core.py.
+        # to these in core.py.
         self.end_offset = value.end_offset
         self.end_linenumber = value.end_lineno
 
@@ -73,6 +79,7 @@ class Statement:
         self.bad_token_index = 0
         self.prev_token = None  # meaningful token preceding bad token
         self.next_token = None  # meaningful token following bad token
+        self.bad_token_comment = None
 
         # SyntaxError produced inside f-strings occasionally require a special treatment
         self.fstring_error = self.filename == "<fstring>" or "f-string" in self.message
@@ -91,8 +98,12 @@ class Statement:
         self.begin_brackets = []  # unclosed ([{  before bad token
         self.end_bracket = None  # single unmatched )]}
 
-        self.first_token = None
-        self.last_token = None
+        self.first_token = None  # meaningful token
+        self.last_token = None  # meaningful token
+
+        # The following is used to indicate the position of ^ and other
+        # symbols when using where()
+        self.location_markers = None
 
         # When using the friendly console (repl), SyntaxError might prevent
         # closing all brackets to complete a statement. Knowing this can be
@@ -196,6 +207,9 @@ class Statement:
                 self.linenumber = 1
         if not source.strip():
             self.source_lines = cache.get_source_lines(self.filename)
+            if self.source_lines == ["\n"]:
+                self.source_lines = ["...\n"] * (self.linenumber - 1)
+                self.source_lines.append(self.bad_line)
             source = "".join(self.source_lines)
             if not source.strip():
                 source = self.bad_line or "\n"
@@ -239,7 +253,6 @@ class Statement:
             self.formatted_partial_source = ""
             return
 
-        last = self.tokens[-1].end_row  # meaningful tokens
         # In some IndentationError cases, and possibly others,
         # Python shows the ^ just before the first token
         # Note that start_row from tokenize and self.offset differ in their origins.
@@ -248,49 +261,102 @@ class Statement:
             self.offset += 1
             self.end_offset += 1
 
-        for index, statement in enumerate(self.all_statements):
-            last_token = statement[-1]
-            if last - last_token.end_row < 5:
-                break
+        lines = self.get_lines_to_show()
+        self.annotate_lines(lines)
 
-        keep = self.all_statements[index:]  # noqa - warnings about index not defined
-        start = keep[0][0].start_row
+    def get_lines_to_show(self):
+        """Restricts the lines of code to be included when showing the location
+        of the error.
+        """
+        self.create_location_markers()
+        first_token = self.all_statements[0][0]
+        lines = get_lines_from_statement(
+            self.statement_tokens, self.linenumber, first_token
+        )
+        if lines:
+            last_linenumber_included, _ = lines[-1]
+            if self.linenumber > last_linenumber_included:
+                # The problem line was an empty line
+                lines.append((last_linenumber_included + 1, ""))
+        return lines
 
-        tokens = []
-        for statement in keep:
-            tokens.extend(statement)
+    def annotate_lines(self, lines):
+        """Adds the caret marks used to show the location of the error"""
+        # Lines is a list of tuples, each tuple has the line number
+        # as a first item, and the text of the line as the second.
+
+        if lines:
+            # Ensures that we align all the lines vertically on a colon
+            # following the line number, even if the number of digits
+            # changes, i.e. we include lines 8, 9, 10, 11.
+            nb_digits = len(str(lines[-1][0]))
+        else:
+            debug_helper.log("No line to annotate in annotate_lines")
+            self.formatted_partial_source = "\n"
+            return
+        no_mark = LINE_NUMBER % nb_digits
+        with_mark = MARKED_LINE_NUMBER % nb_digits if len(lines) > 1 else no_mark
+        leading_spaces = " " * (len(LINE_NUMBER % lines[-1][0]) - 3)
 
-        partial_source = token_utils.untokenize(tokens)
-        lines = partial_source.split("\n")
-        nb_lines = len(lines)
         new_lines = []
-        nb_digits = len(str(start + nb_lines))
-        no_mark = "       {:%d}: " % nb_digits
-        with_mark = "    -->{:%d}: " % nb_digits
-
-        offset_mark = " " * (8 + nb_digits + self.offset) + "^"
-
-        marked = False
-        for i, line in enumerate(lines, start):
-            if i == self.linenumber:
+        for i, line in lines:
+            if i in self.location_markers:
                 num = with_mark.format(i)
-                new_lines.append(num + line.rstrip())
-                new_lines.append(offset_mark)
-                marked = True
-            elif marked:
-                if not line.strip():  # do not add empty lines after problem line
-                    break
-                num = no_mark.format(i)
-                new_lines.append(num + line.rstrip())
-            elif i > self.linenumber + 5:
-                # avoid printing to the end of the file if we have an unclosed bracket
-                break
+                new_lines.append(num + line)
+                new_lines.append(leading_spaces + self.location_markers[i])
             else:
                 num = no_mark.format(i)
-                new_lines.append(num + line.rstrip())
+                new_lines.append(num + line)
 
         self.formatted_partial_source = "\n".join(new_lines)
 
+    def create_location_markers(self):
+        # In some cases, the location markers are determined while analysing
+        # the statement to find the cause.
+        if self.location_markers is not None:
+            return
+        # We generally go with the information obtained by Python.
+        # although, sometimes it might be off by 1.
+        nb_carets = 1  # Python default
+        diff = self.bad_token.start_col - self.offset
+        self.offset += diff
+        if self.end_offset is not None:
+            self.end_offset += diff
+
+        continuation = ""
+        if self.end_offset is not None:
+            if (
+                self.end_offset > self.offset
+                and self.linenumber == self.end_linenumber
+                and self.highlighted_tokens is not None
+                and len(self.highlighted_tokens) > 1
+            ):
+                if (
+                    self.end_offset > self.last_token.end_col
+                    and self.linenumber == self.last_token.start_col
+                ):  # a comment is highlighted
+                    self.end_offset = self.last_token.end_col
+                    continuation = "~~-"
+                nb_carets = self.end_offset - self.offset
+            elif self.linenumber != self.end_linenumber:
+                continuation = "-->"
+
+        if nb_carets == 1 and not continuation:
+            nb_carets = len(self.bad_token.string)
+        # If the bad_token was a comment, it means that something was missing
+        # between the comment and the last meaningful token
+        if self.bad_token_comment is not None:
+            offset_mark = (
+                " " * self.offset
+                + "-" * nb_carets
+                + "^"
+                * max((self.bad_token_comment.start_col - self.bad_token.end_col), 1)
+                + "-->"
+            )
+        else:
+            offset_mark = " " * self.offset + "^" * nb_carets + continuation
+        self.location_markers = {self.linenumber: offset_mark}
+
     def obtain_statement(self, source_tokens):
         """This method scans the source searching for the statement that
         caused the problem. Most often, it will be a single line of code.
@@ -315,24 +381,33 @@ class Statement:
         # Some tokens cannot occur within brackets; if they are indicated as being
         # the offending token, it might be because we have an unclosed bracket.
         should_begin_statement = [
+            "assert",
             "async",
             "await",
+            "break",
             "class",
+            "continue",
             "def",
-            "return",
+            "del",
             "elif",
-            "import",
-            "try",
             "except",
             "finally",
+            "global",
+            "import",  # cannot write: from (x import ...
+            "nonlocal",
+            "pass",
+            "raise",
+            "return",
+            "try",
             "with",
             "while",
             "yield",
         ]
 
+        last_line_to_include = self.linenumber
         for token in source_tokens:
             if (
-                token.start_row > self.linenumber
+                token.start_row > last_line_to_include
                 and not continuation_line
                 and not self.statement_brackets
             ):
@@ -343,7 +418,7 @@ class Statement:
             # A new statement will typically start on a new line and will be
             # preceded by valid statements.
 
-            # An initial version was based on the assumption that any semi-colon
+            # An initial version was based on the assumption that any semicolon
             # would be used correctly and would indicate the end of a statement;
             # however, I am guessing that it more likely indicates
             # a typo, and that the user wanted to write a comma or a colon, so I
@@ -351,11 +426,18 @@ class Statement:
             if token.start_row > previous_row:
                 if previous_token is not None:
                     continuation_line = previous_token.line.endswith("\\\n")
-                if token.start_row <= self.linenumber and not self.statement_brackets:
+                if (
+                    token.start_row <= last_line_to_include
+                    and not self.statement_brackets
+                ):
                     if self.statement_tokens:
                         self.all_statements.append(self.statement_tokens[:])
                     self.statement_tokens = []
                     self.begin_brackets = []
+                if token.start_row > last_line_to_include and self.statement_brackets:
+                    # The statement continues beyond the line with the error, so
+                    # we make sure to include this additional line.
+                    last_line_to_include = token.start_row
                 previous_row = token.start_row
 
             self.statement_tokens.append(token)
@@ -377,13 +459,14 @@ class Statement:
                 ):
                     self.highlighted_tokens.append(token)
             elif (
-                token.start_row == self.linenumber
+                token.start_row == last_line_to_include
                 and token.start_col <= self.offset <= token.end_col
                 and self.bad_token is None
                 and token.string.strip()
             ):
                 self.bad_token = token
                 if self.bad_token.is_comment():
+                    self.bad_token_comment = self.bad_token
                     self.bad_token = self.prev_token
                 if self.highlighted_tokens is not None:
                     self.highlighted_tokens.append(self.bad_token)
@@ -396,7 +479,11 @@ class Statement:
 
             previous_token = token
 
-            if self.bad_token in should_begin_statement and self.statement_brackets:
+            if (
+                self.bad_token in should_begin_statement
+                and self.bad_token != self.statement_tokens[0]
+                and self.statement_brackets
+            ):
                 break  # we almost certainly have an unclosed bracket
             # Note: '' in 'any string' == True
             # careful to not accidentally include null strings as brackets
@@ -441,3 +528,34 @@ class Statement:
                 self.bad_token_index = index
             index += 1
         return tokens
+
+
+def get_lines_from_statement(tokens, linenumber, prev_token):
+    lines = []
+    end_docstring = False
+    for token in tokens:
+        current_linenumber = token.start_row
+        current_line = token.line.rstrip()
+        if lines and prev_token.end_row == current_linenumber:
+            continue
+        if linenumber < current_linenumber and token == tokens[0]:
+            break
+        if linenumber - current_linenumber < 5:
+            if end_docstring == current_line:
+                end_docstring = False
+                continue
+            if "\n" in current_line:
+                text = current_line.split("\n")  # handle docstring
+                for line in text:
+                    lines.append((current_linenumber, line))
+                    current_linenumber += 1
+                end_docstring = text[-1]
+                current_linenumber -= 1
+            else:
+                lines.append((current_linenumber, current_line))
+        else:
+            if "\n" in current_line:
+                text = current_line.split("\n")
+                end_docstring = text[-1]
+        prev_token = token
+    return lines
