@@ -60,6 +60,7 @@ class RuntimeMessageParser:
             cause = self.current_parser(value_or_message, frame, tb_data)
             if cause:
                 return cause
+        debug_helper.log_2(str(value_or_message))
         return {"cause": no_information(), "suggest": unknown_case()}
 
 
@@ -135,6 +136,11 @@ def get_similar_words(word_with_typo: str, words: Iterable[str]) -> List[str]:
     for distance in range(1, max_dist + 1):
         if distance in similar_words and similar_words[distance]:
             similar.extend(similar_words[distance])
+    if not similar:  # example PI -> pi
+        if word_with_typo.lower() in words:
+            similar.append(word_with_typo.lower())
+        elif word_with_typo.upper() in words:
+            similar.append(word_with_typo.upper())
     return similar
 
 
@@ -266,3 +272,17 @@ def get_bad_statement(tb_data: "TracebackData") -> str:
         elif hasattr(tb_data, "bad_line"):
             return tb_data.bad_line
         return ""
+
+
+def to_code_block(code: str) -> str:
+    """Takes some code and indent it with an added empty line at the top
+    and bottom so that it is usable as a Markdown code block without
+    showing the triple backquotes.
+    """
+    indent = " " * 4
+    new_lines = ["\n"]
+    lines = code.split("\n")
+    for line in lines:
+        new_lines.append(f"{indent}{line}")
+    new_lines.append("\n")
+    return "\n".join(new_lines)
